#include "onixapi.h"
#include <stdio.h>
#include <string.h>
#include <time.h>


/*

  This is a simple test program for Onix that should be able to be run unmodified on most
  platforms.  It includes a simple text parser and a "choose a number" style interface.
  Most all of the interface calls are mapped directly to the Onix API which should give
  you a good feel as to how the API is used and can be used for testing purposes.

  The text parser is designed to take an ASCII text file and parse it into records and
  words.  The default record delimiter is an ASCII (decimal) 16.  For test files, it you
  can get text files at http://www.gutenberg.net.  Putting an ASCII (decimal) 16 between
  paragraphs and saving the files to a plain text file is all that is needed to prepare
  them for use with this test program.

  Some simple calling sequences are as follows:


  CREATE AN INDEX, INDEX A TEXT FILE, CLOSE INDEX

  1)	Create Index
			2) Word Mode
			Distributed Index: n
			Store Text Data : y
			Store Record ID: n
  2)	Open Index
			<FileName>
  3)	Start Indexing Session
  4)	Index Text File
			<FileName>
			Store Text Data: y
			Store Record ID: n
  3)	End Indexing Session
  2)	Close Index.


  OPEN AN INDEX, CONDUCT A SEARCH, PRINT OUT THE FIRST FOUR RECORD NUMBERS,  CLOSE INDEX

  2)	Open the index
			<FileName>
  5)	Start Retrieval Session
  7)	Perform Query
			Position To Store Results (1-50) : 1
			Query: cat & dog
  11)	Current Hit
			Vector To Look at: 1
  12)	Next Hit
			Vector To Look at: 1
  12)	Next Hit
			Vector To Look at: 1
  12)	Next Hit
			Vector To Look at: 1
  5)	End retrieval Session
  2)	Close Index.



  OPEN THE INDEX, AND RETRIEVE THE TEXT ASSOCIATED WITH A RECORD.

  2)	Open the index
			<FileName>
  5)	Start Retrieval Session
  21)	Get the record text
			Record of text to retrieve: 21
  22)	Get more record text (Perform this until finished.)			
  5)	End Retrieval Session
  2)	Close the Index



  OPEN THE INDEX AND DELETE A RECORD

  2)	Open the index
			<FileName>
  5)	Start Retrieval Session
  14)	Delete Record
			Record Number: 65004
  5)	End Retrieval Session
  2)	Close Index.
			

*/




/*
This table tells us which characters are letters and belong to a word and which characters 
are whitespace or need to be ignored.  It is easy to modify this table to change which characters
are part of a word or not.  This can be done by changing the values of the #defines below and by
modifying the table directly.  Note, the table has 10 characters per row making it easy to find
any given character's entry.

  NOTE: The current table uses the character with the decimal value of 16 as a record delimiter.  
  This can be changed by moving (or removing) the INCREMENTREC #define or changing its value.
*/

#define		CHARACTER		(1)
#define		WHITESPACE		(2)
#define		NUMBER			(3)
#define		INCREMENTREC	(4)

#define		EAT_WHITESPACE	(1)
#define		EAT_CHARACTERS	(2)


unsigned char character_type[256]=
{
		  WHITESPACE,
			  WHITESPACE,WHITESPACE,WHITESPACE,WHITESPACE,WHITESPACE,WHITESPACE,WHITESPACE,WHITESPACE,WHITESPACE,WHITESPACE,
			  WHITESPACE,WHITESPACE,WHITESPACE,WHITESPACE,WHITESPACE,INCREMENTREC,WHITESPACE,WHITESPACE,WHITESPACE,WHITESPACE,
			  WHITESPACE,WHITESPACE,WHITESPACE,WHITESPACE,WHITESPACE,WHITESPACE,WHITESPACE,WHITESPACE,WHITESPACE,WHITESPACE,
			  WHITESPACE,WHITESPACE,WHITESPACE,WHITESPACE,WHITESPACE,WHITESPACE,WHITESPACE,WHITESPACE,WHITESPACE,WHITESPACE,
			  WHITESPACE,INCREMENTREC,WHITESPACE,WHITESPACE,WHITESPACE,WHITESPACE,WHITESPACE,NUMBER,NUMBER,NUMBER,
			  NUMBER,NUMBER,NUMBER,NUMBER,NUMBER,NUMBER,NUMBER,WHITESPACE,WHITESPACE,WHITESPACE,
			  WHITESPACE,WHITESPACE,WHITESPACE,WHITESPACE,CHARACTER,CHARACTER,CHARACTER,CHARACTER,CHARACTER,CHARACTER,
			  CHARACTER,CHARACTER,CHARACTER,CHARACTER,CHARACTER,CHARACTER,CHARACTER,CHARACTER,CHARACTER,CHARACTER,
			  CHARACTER,CHARACTER,CHARACTER,CHARACTER,CHARACTER,CHARACTER,CHARACTER,CHARACTER,CHARACTER,CHARACTER,
			  WHITESPACE,WHITESPACE,WHITESPACE,WHITESPACE,WHITESPACE,WHITESPACE,CHARACTER,CHARACTER,CHARACTER,CHARACTER,
			  CHARACTER,CHARACTER,CHARACTER,CHARACTER,CHARACTER,CHARACTER,CHARACTER,CHARACTER,CHARACTER,CHARACTER,
			  CHARACTER,CHARACTER,CHARACTER,CHARACTER,CHARACTER,CHARACTER,CHARACTER,CHARACTER,CHARACTER,CHARACTER,
			  CHARACTER,CHARACTER,WHITESPACE,WHITESPACE,WHITESPACE,WHITESPACE,WHITESPACE,WHITESPACE,WHITESPACE,WHITESPACE,
			  WHITESPACE,WHITESPACE,WHITESPACE,WHITESPACE,WHITESPACE,WHITESPACE,WHITESPACE,WHITESPACE,WHITESPACE,WHITESPACE,
			  WHITESPACE,WHITESPACE,WHITESPACE,WHITESPACE,WHITESPACE,WHITESPACE,WHITESPACE,WHITESPACE,WHITESPACE,WHITESPACE,
			  WHITESPACE,WHITESPACE,WHITESPACE,WHITESPACE,WHITESPACE,INCREMENTREC,WHITESPACE,WHITESPACE,WHITESPACE,WHITESPACE,
			  WHITESPACE,WHITESPACE,WHITESPACE,WHITESPACE,WHITESPACE,WHITESPACE,WHITESPACE,WHITESPACE,WHITESPACE,WHITESPACE,
			  WHITESPACE,WHITESPACE,WHITESPACE,WHITESPACE,WHITESPACE,WHITESPACE,WHITESPACE,WHITESPACE,WHITESPACE,WHITESPACE,
			  WHITESPACE,WHITESPACE,WHITESPACE,WHITESPACE,WHITESPACE,WHITESPACE,WHITESPACE,WHITESPACE,WHITESPACE,WHITESPACE,
			  WHITESPACE,WHITESPACE,WHITESPACE,WHITESPACE,WHITESPACE,WHITESPACE,WHITESPACE,WHITESPACE,WHITESPACE,WHITESPACE,
			  WHITESPACE,WHITESPACE,WHITESPACE,WHITESPACE,WHITESPACE,WHITESPACE,WHITESPACE,WHITESPACE,WHITESPACE,WHITESPACE,
			  WHITESPACE,WHITESPACE,WHITESPACE,WHITESPACE,WHITESPACE,WHITESPACE,WHITESPACE,WHITESPACE,WHITESPACE,WHITESPACE,
			  WHITESPACE,WHITESPACE,WHITESPACE,WHITESPACE,WHITESPACE,WHITESPACE,WHITESPACE,WHITESPACE,WHITESPACE,WHITESPACE,
			  WHITESPACE,WHITESPACE,WHITESPACE,WHITESPACE,WHITESPACE,WHITESPACE,WHITESPACE,WHITESPACE,WHITESPACE,WHITESPACE,
			  WHITESPACE,WHITESPACE,WHITESPACE,WHITESPACE,WHITESPACE,WHITESPACE,WHITESPACE,WHITESPACE,WHITESPACE,WHITESPACE,
			  WHITESPACE,WHITESPACE,WHITESPACE,WHITESPACE,WHITESPACE
};


/* 
The character_convert array is used to quickly convert upper case characters to 
lower case. The array must be initialized by a call to init_character_convert() 
before it is usable.
*/

char character_convert[256];


/*
This function initilizes the character_convert array.  First it initializes the table 
so that every index reflects its corresponding character.  Thus character_convert['a'] == 'a'.
Next we normalize all the array indexes that correspond to upper case characters so that they
reflect their lowercase counterparts.
*/

void init_character_convert()
{
		  int counter;
		  
		  for (counter=0;counter<=128;counter++)
			  character_convert[counter]=counter;
		  
		  for (counter=65;counter<=90;counter++)
			  character_convert[counter]=counter+32;
}

/*
This is the word structure that is used to pass words into the indexing engine.
The only entries in the structure that currently need to be filled out is the word
itself (in the Word buffer) and the Length entry.
*/
WordTypeT Word;



/*
This is the scratch buffer which is used to build up the text for
the record data if it is going to be stored.  This buffer assumes
that the record is under 16K in size.  If it is larger than this,
then we will only store the first 16K. The Buff offset keeps track of 
where we are in the scratch buffer.  The StoreText is our flag telling
us if we are storing text or not.
*/
size_t		BuffOffset = 0;
UCharT		ScratchBuff[16284];  
BooleanT	StoreText = BooleanFalse;

/* 
Record ID Stuff....
These are the flags and buffers for storing the record IDs.
The record IDs here are generated inside the scan() function
and are quite simple minded.  The record ID is simply the record
number with a "RecID prepended.  I.e, the record ID for record 
number 35 is "RecID35"

Record IDs can either be fixed or variable length.  These test
routines allow either to be defined.
*/
UCharT		RecordIDBuff[80];
ULongT		RecordIDLength = 0;
ULongT		FixedLengthRecordIDFlag = 0;
ULongT		StoreRecordIDFlag = 0;
BooleanT	IndexRecordIDsFlag = BooleanFalse;

// The current record number.  Use for testing or debugging.
unsigned long CurrentRecord;


/*
This is the main function used to parse a file.  This is a simple
parser which considers a word to be a string of characters boardered
by non-characters.  

Two tables are used for the indexing process.  The first is the character_type
array which defines the type each byte read in falls into (character,
whitespace, record delimiter, number, etc.)  The second is the character_convert
array which is used to transform upper case characters to lower case characters.

scan() also stores the record text if chosen and also can store a record ID if
desired.

As you might notice, the parser simply switches state between characters, whitespace,
and record delimiters.  While a simple parser, it is extremly fast.

*/

void scan(FILE *Infile, OnixIndexingEngineT Indexer)
{
	char  	ch;
	int   	place = 0;
	size_t	BuffOffset = 0;
	int   	state = EAT_WHITESPACE;
	unsigned long RecordNum = 1, WordNum = 1;
	StatusCodeT Status;
		  
		  
	while (EOF != (ch=getc(Infile)) )	 {
			  
		if(StoreText == BooleanTrue && BuffOffset < 16284) {
			ScratchBuff[BuffOffset] = ch;
			BuffOffset ++;
			if(BuffOffset == 4096) {
				ixStoreRecordData(Indexer,ScratchBuff,BuffOffset,&Status);
				BuffOffset = 0;
			}
		}
			  
		Re_do:    
		switch (state)
		  {
					 case EAT_CHARACTERS: 
						 switch (character_type[ch])  {
							case CHARACTER:		Word.Word[place++]=character_convert[ch];
												break;
						 
							case WHITESPACE:	state=EAT_WHITESPACE;
												Word.Word[place]=0;
												Word.Length=place;
												ixIndexWord(Indexer,&Word,&Status);
												WordNum++;
						 
												place=0;
												goto Re_do;
						 
							case INCREMENTREC:	state=EAT_WHITESPACE;
												Word.Word[place]=0;
												Word.Length=place;
												ixIndexWord(Indexer,&Word,&Status);
												if(StoreText == BooleanTrue) {
													ixStoreRecordData(Indexer,ScratchBuff,BuffOffset,&Status);
												}
						 
												if(StoreRecordIDFlag) {
													sprintf((char *)RecordIDBuff,"RecID%ld",RecordNum,&Status);
													if(FixedLengthRecordIDFlag) {
														ixProcessRecordID(Indexer,RecordIDBuff,RecordIDLength,BooleanTrue, IndexRecordIDsFlag,&Status);
													}
													else {
														ixProcessRecordID(Indexer,RecordIDBuff,strlen((const char *)RecordIDBuff),BooleanTrue, IndexRecordIDsFlag, &Status);
													}
												}
												BuffOffset = 0;
												ixIncrementRecord(Indexer,&Word,&Status);
												RecordNum ++;
												WordNum = 1;
												place=0;
												CurrentRecord++;
												break;
												default: break;
						};
						 break;
						 
					 case EAT_WHITESPACE: 
							switch (character_type[ch])  {
								case CHARACTER: state=EAT_CHARACTERS;
												place=0;
												goto Re_do;
						 
								case WHITESPACE:break;
						 
								case INCREMENTREC:state=EAT_WHITESPACE;
												place=0;
												if(StoreText == BooleanTrue) {
													ixStoreRecordData(Indexer,ScratchBuff,BuffOffset,&Status);
												}
												if(StoreRecordIDFlag) {
													sprintf((char *)RecordIDBuff,"RecID%ld",RecordNum);
													if(FixedLengthRecordIDFlag) {
														ixProcessRecordID(Indexer,RecordIDBuff,RecordIDLength,BooleanTrue, IndexRecordIDsFlag, &Status);
													}
													else {
														ixProcessRecordID(Indexer,RecordIDBuff,strlen((const char *)RecordIDBuff),BooleanTrue, IndexRecordIDsFlag, &Status);
													}
												}
												BuffOffset = 0;
												ixIncrementRecord(Indexer,&Word,&Status);
												CurrentRecord++;
												RecordNum ++;
												WordNum = 1;
												break;
												default: break;
							};
							break;
		};  // end of switch (state)
	}  // end of while
		  
	
	/*
	Store any last data that we may have not already 
	stored before we bail.
	*/
	if(StoreText == BooleanTrue) {
		if(BuffOffset > 0) {
			ixStoreRecordData(Indexer,ScratchBuff,BuffOffset,&Status);
		}
	}
} // End of function


/*
These are the sizes of the different buffers that are used during the indexing
process.  Onix is extremely fast even when supplied with very small buffers.
In fact, we recommend that the three buffers be 60K in size.
*/

#define BUFFER1SIZE 60000
#define BUFFER2SIZE 60000
#define BUFFER3SIZE 60000

int main() {	
	IndexModeT  IndexingMode = RecordMode;
	ULongT		IndexMode = 1;
	/* The three buffers that are used for various processes during the indexing process */
	char *Buffer1 = NULL;	char *Buffer2 = NULL;	char *Buffer3 = NULL;
	/* The index manager that is used to open, manage, and search the index. */
	OnixIndexManagerT   IndexManager;
	/* The indexer that is used during an indexing session. */
	OnixIndexingEngineT  Indexer;
	/* Array of result vectors.  Since it is an array, you can have more
	than one result vector floating around which you can traverse or perform
	boolean operations on. */
	OnixQueryVectorT Vector[51];
	/* Various buffers for getting the user's response as well
	as getting the query and performing the query conversions. 	*/
	char	Choice[2048], TextQueryString[2048],HexQueryString[2048], RankedHexQueryString[2048];
	/* This buffer is used to retrieve stored text data and record IDs. */
    char	ScratchBuff[80];
	/* Various status variables. */
	int		Result, Result2, VecPos1, VecPos2, VecPos3;
	/* The file pointer that we are using to read the text to be indexed in through. */
	FILE	*Infile = NULL;
	/* The filename of various files we might be working with.  Note, it is only 80 characters long. */
	char	FileName[80];
	int		BuffSize, Counter;
	StatusCodeT Status;
	/* Variable to keep track of the various record and word numbers we are working with.
	as well as how many search results we have. */
	RecordNumT	RecordNum, WordNum, CountNum, NumRecords, NumWords, NumbHits;
	/* Flags as to whether a search term is found or whether we have a distributed index or not. */
	BooleanT	Found, DistributedIndex = BooleanFalse;
	size_t		Length;
	/* This is the cursor that keeps track of where we are in the dictionary. */
	DictionaryCursorT Cursor;
	/* The data cursor that is used to retrieve data from the index. */
	OnixDataCursorT	DataCursor;
	/* The length of the temporary files that are generated during indexing. */
	ULongT		TempFileLength;
	/* These are the creation params that are used to create indexes. */
	OnixIndexCreationParamsT IndexCreationParams;
   /* The weight given to a given record for ranking purposes. */
   FloatT		HitWeight;

	/* Flags to manage where we are in the interface. */
	BooleanT	IndexingSessionOngoingFlag = BooleanFalse;
	BooleanT RetrievalSessionOngoingFlag = BooleanFalse;
	BooleanT	OpenIndexFlag = BooleanFalse;
	
	/* This times out in November 2000 */
	IndexManager = ixCreateIndexManager(0x54F30A78L, 0xE5A60978, &Status);
	if(Status < 0) {
		printf("\n%d",Status);
	}

	/* Initialize the various buffers used during indexing. */
	Buffer1 = (char *) malloc(BUFFER1SIZE);
	Buffer2 = (char *) malloc(BUFFER2SIZE);
	Buffer3 = (char *) malloc(BUFFER3SIZE);
	if(Buffer1 == NULL || Buffer2 == NULL || Buffer3 == NULL) {
		if(Buffer1 != NULL)
			free(Buffer1);
		if(Buffer2 != NULL)
			free(Buffer2);
		if(Buffer3 != NULL)
			free(Buffer3);
		printf("\nError allocating memory...");
		exit(1);
	}
	
	/* Initialize the character convert arrary. */
	init_character_convert();

	
	for(;;) {
		printf("\n\n1)  Create Index                17) Output Stats File");
		printf("\n2)  Open / Close Index            18) Get Current Query Vector");
		printf("\n3)  Start / End Indexing Session  19) Get Record ID");
		printf("\n4)  Index File                    20) Find ID's Record Num");
		printf("\n5)  Start / End RetrievalSession  21) Get Record Text");
		printf("\n6)  Set Index Output File Name    22) Get More Record Text");
		printf("\n7)  Query The Index               23) Get Error Message");
		printf("\n8)  Examine Wordlist              24) Get Temp Disk Space Usage.");
		printf("\n9)  Num. Recs In Index             ");
		printf("\n10) Rewind Vector                  ");
		printf("\n11) Get Current Hit                ");
		printf("\n12) Get Next Hit                   ");
		printf("\n13) Get Number Of Hits             ");
		printf("\n14) Delete Record                  ");
		printf("\n15) Undelete Record                ");
		printf("\n16) Is Record Deleted              ");
		printf("\n0)  Quit \n: ");		
		gets(Choice);
		Result = atoi(Choice);
		switch(Result) {
		case 1:     
			/*
				Before we create the index, we need to get certain
				data from the user.  We need to find out :

				1) What style index is needed.
				2) Whether the index will be distributed across multiple files.
				2) Whether the index is going to store data with 
				   each record.
				3) Whether there will be a record ID with each record.
				4) The filename for the index.
			*/


			printf("\n\t1) Record Mode");
			printf("\n\t2) Word/Record Mode");
			printf("\n\t3) Idf Mode");
			printf("\n\t:");
			gets(Choice);
			IndexingMode = (IndexModeT) atoi(Choice);
			/*	We are going to use ixCreateIndexEx so as to handle the
				additional options.  So what we need to do is create the
				index creation params object. So we can set the various
				flags and variables.
			*/
			IndexCreationParams = ixCreateIndexCreationParams(&Status);
			/*	Set the indexing mode. */
			ixSetIndexCreationParams(IndexCreationParams,ixSetIndexMode,&IndexingMode);
         /* Set the ranking mode. */
         if(IndexingMode == IDFMode || IndexingMode == WordMode)
         	ixSetIndexCreationParams(IndexCreationParams,ixSetRankingMethodTwo,NULL);
			printf("\nDistributed Index? (Y/N) :");
			gets(Choice);
			/*	Set whether we are going to distribute the index across multiple files.*/
			if(strchr(Choice,'Y') || strchr(Choice,'y')) {
				DistributedIndex = BooleanTrue;
				ixSetIndexCreationParams(IndexCreationParams,ixSetDistributedIndex,NULL);
			}
			else {
				DistributedIndex = BooleanFalse;
				ixSetIndexCreationParams(IndexCreationParams,ixSetSinglePointIndex,NULL);
			}
			printf("\nStore Text Data? (Y/N)");
			gets(Choice);
			/* 
				Set whether we are going to store data with each record.  Since the parser
				is set up to store the record text when this is set, we will use a variable
				length record option since the records are most likely of variable length.
			*/
			if(strchr(Choice,'Y') || strchr(Choice,'y')) {
				StoreText = BooleanTrue;
				ixSetIndexCreationParams(IndexCreationParams,ixSetVariableLengthRecordInfo,NULL);
				ixSetIndexCreationParams(IndexCreationParams,ixSet32BitRecordInfoOffset,NULL);

			}
			else
				StoreText = BooleanFalse;
			
			printf("\nStore Record ID ? (y/n)");
			gets(Choice);
			/* 
				Set whether there will be  record ID stored with each record.
				If so, then there is a choice between a fixed or variable length
				record ID which gets set also.
			*/
			if(strchr(Choice,'y') || strchr(Choice,'Y') ) {
				printf("\nFixed Length ID ? (y/n)");
				gets(Choice);
				if(strchr(Choice,'y') || strchr(Choice,'Y')) {
					printf("\nRecord ID Length :");
					gets(Choice);
					RecordIDLength = atoi(Choice);
					ixSetIndexCreationParams(IndexCreationParams,ixSetFixedLengthRecordID,&RecordIDLength);
					FixedLengthRecordIDFlag = 1;
					StoreRecordIDFlag = 1;
				}
				else {
					ixSetIndexCreationParams(IndexCreationParams,ixSetVariableLengthRecordID,NULL);
					StoreRecordIDFlag = 1;
				}
				ixSetIndexCreationParams(IndexCreationParams,ixSetRecordID32BitOffset,NULL);
			}

			printf("\nIndex Name :");
			gets(Choice);
			/*	Set the index's filename. */
			ixSetIndexCreationParams(IndexCreationParams,ixSetIndexFileName,Choice);
			/*	Now that we have done all this, we can go ahead and create the index. */
			ixCreateIndexEx(IndexManager,IndexCreationParams,&Status);
			/*	Delete the index creation params now that we don't need them anymore. */
			ixDeleteIndexCreationParams(IndexCreationParams);
			if(Status < 0)
				printf("\nError Creating Index: %d",Status);
			break;
		case 2:
			/*
				We are going to be either opening or closing
				the index.  We use the OpenIndexFlag to tell 
				whether there is an open index or not.  If there
				isn't then we open one, if there is then we close it.
			*/
			if(OpenIndexFlag == BooleanTrue) {
				if(IndexManager != NULL)
					/* Close the index. */
					ixCloseIndex(IndexManager,&Status);
				if(Status < 0) 
					printf("\nError Closing Index: %d",Status);
				else
					printf("\nIndex Closed Successfully.");
				OpenIndexFlag = BooleanFalse;
				break;
			}
			
			printf("\nName of Index to Open :");
			fgets(Choice,80,stdin);
			strtok(Choice,"\n\r");
			/* Open the index. */
			ixOpenIndex(IndexManager,Choice,&Status);
			if(Status < 0)
				printf("\nFailure opening index: %d",Status);
			else
				OpenIndexFlag = BooleanTrue;
			break;
			
			
		case 3:
			/*
				When indexing, you need to start an indexing session
				before performing the indexing and end the indexing
				session when you are finished.

				This does both.  If an indexing session is already
				in progress, it ends it.  If not, it starts a new
				indexing session.

				In order for an indexing session to work, then we
				must have an index both created and opened.  (Or
				just opened in the case of a pre-existing index
				that we are going to add data to.)
			*/
			if(IndexingSessionOngoingFlag == BooleanTrue) {
				ixEndIndexingSession(IndexManager,Indexer,&Word, &Status);
				if(Status < 0)
					printf("\nError Ending Indexing Session. %d",Status);
				else
					printf("\nIndexing Session Successfully Ended.");
				IndexingSessionOngoingFlag = BooleanFalse;
				break;
			}
			Indexer = ixStartIndexingSession(IndexManager,&Status);
			
			if(Indexer == NULL || Status < 0) {
				printf("\nError Starting Indexing Session. %d",Status);
			}
			else {
				printf("\nIndexing Session Successfully Started.");
				IndexingSessionOngoingFlag = BooleanTrue;
			}
			break;
			
			
		case 4:	 
			/*
				We are going to open the file and then
				call the parser to parse it and close the
				file here.

				It is important to point out that the parser
				is set up to break the text up into records
				based on a single character delimiter.  The
				default is ASCII (decimal) 16.

				We also need to ask about whether we are going
				to store the text into the index as well as 
				whether we are going to have a record ID for each
				record.  If we are going to do these, we need
				to make sure the index is created in such a way
				that it is compatable with these options.

				When we are done parsing the text, we will close
				the file.
			*/
			printf("\nName of File to Scan :");
			gets(FileName);
			Infile = fopen(FileName,"rt");
			if(Infile == NULL) {
				printf("\nError opening file");
				break;
			}
			else {
				printf("\nFile opened successfully");
				BuffSize = 4096;
				setvbuf(Infile,NULL,_IOFBF,BuffSize);
			}
			/*	
				Find out whether we are going to store the 
				text data into the index.
			*/
			
			IndexRecordIDsFlag = BooleanFalse;
			StoreRecordIDFlag = 0;
			printf("\nStore Text Data? (Y/N)");
			fgets(Choice,80,stdin);
			if(strchr(Choice,'Y') || strchr(Choice,'y')) {
				StoreText = BooleanTrue;
			}
			else
				StoreText = BooleanFalse;
			

			/*
				Find out whether we are going to have a record
				ID for each record.  We also have the option
				of indexing the record ID or not so we can map
				from a record ID to a record number if needed.
				(Indexing the record IDs expands the index slightly.)
			*/
			printf("\nStore Record ID ? (y/n) : ");
			fgets(Choice,80,stdin);
			if(strchr(Choice,'y') || strchr(Choice,'Y') ) {
				printf("\nFixed Length ID ? (y/n) : ");
				fgets(Choice,80,stdin);
				if(strchr(Choice,'y') || strchr(Choice,'Y')) {
					printf("\nRecord ID Length :");
					fgets(Choice,80,stdin);
					RecordIDLength = atoi(Choice);
					FixedLengthRecordIDFlag = 1;
					StoreRecordIDFlag = 1;
				}
				else {
					StoreRecordIDFlag = 1;
				}
				printf("\nIndex Record ID ? (y/n) : ");
				fgets(Choice,80,stdin);
				if(strchr(Choice,'y') || strchr(Choice,'Y')) {
					IndexRecordIDsFlag = BooleanTrue;
				}
				
			}
			
			/*
				Now that we have the parser all set up, we go
				ahead and parse the text.
			*/
			scan(Infile, Indexer);
			printf("\nDone\a...");
			
			/* Close the text file. */
			fclose(Infile);
			Infile = NULL;
			printf("\nFile Successfully Closed");
			break;
			
		case 5:     
			/*
				If we are going to be doing some searching, we
				need to start a retrieval session.  (This causes
				all parts of the index to be loaded into memory.)

				If we are done searching, then we end the retrieval
				session.

				We decide which to do by watching the flag:
				RetrievalSessinoOngoingFlag.
			*/
			if(RetrievalSessionOngoingFlag == BooleanTrue ) {
				/*	End the retrieval session.	*/
				ixEndRetrievalSession(IndexManager,&Status);
				if(Status < 0) {
					printf("\nError Ending Retrieval Session :%d",Status);
				}
				RetrievalSessionOngoingFlag = BooleanFalse;
				break;
			}
			
			/*	Start the retrieval session. */
			ixStartRetrievalSession(IndexManager,&Status);
			if(Status < 0) {
				printf("\nError Starting Retrieval Session: %d",Status);
			}
			else
				RetrievalSessionOngoingFlag = BooleanTrue;
			break;
			
		case 6:   
			/*
				If we have a distributed index then we need to set
				the filename as to where we want to store the current
				index data to.  This must be done sometime during
				the indexing and before the call to ixEndIndexingSession()
				or ixFinalProcessIndex().
				
			*/
			printf("\nIndex File Name :");
			gets(Choice);
			ixSetFinalIndexDataPosition(Indexer,Choice,&Status);
			if(Status < 0) {
				printf("\nError: %d",Status);
			}
			break;
			
		case 7:
			/*
				We are going to conduct a search.  We'll store the results
				in the result vector array allowing us to deal with multiple
				result lists at once.
			*/
			printf("\nPlace Result Vector Where? (1-50) :");
			gets(Choice);
			VecPos1 = atoi(Choice);
			printf("\nBoolean Query :");
			fgets(TextQueryString,80,stdin);
			strtok(TextQueryString,"\n\r");
			/*	Convert the query from the user's form to the standardized hex form. */
			ixConvertQuery((UCharT *)TextQueryString,(UCharT *)HexQueryString);
         printf("\nRanked Query :");
			fgets(TextQueryString,80,stdin);
			strtok(TextQueryString,"\n\r");
         /*	Convert the query from the user's form to the standardized hex form. */
			ixConvertQuery((UCharT *)TextQueryString,(UCharT *)RankedHexQueryString);

			/*	Perform the actual query and store the results in the results array.*/
         if(strlen(RankedHexQueryString) == 0)
				Vector[VecPos1] = ixProcessQuery(IndexManager,NULL,(UCharT *)HexQueryString,&Status);
         if(strlen(HexQueryString) == 0)
         	Vector[VecPos1] = ixProcessQuery(IndexManager,(UCharT *)RankedHexQueryString,NULL,&Status);
         if(strlen(RankedHexQueryString) != 0 && strlen(HexQueryString) != 0)
         	Vector[VecPos1] = ixProcessQuery(IndexManager,(UCharT *)RankedHexQueryString,(UCharT *)HexQueryString,&Status);
			break;

		case 8:
			/*
				Now we are going to navigate the wordlist.
				Before we can do that, we must call FindWord on a word to
				position the wordlist cursor at a position within the wordlist.
			*/
			printf("\n1)  Find Word");
			printf("\n2)  Current Word");
			printf("\n3)  Previous Word");
			printf("\n4)  Next Word\n:");
			fgets(Choice,80,stdin);
			strtok(Choice,"\n\r");
			Result2 = atoi(Choice);
			switch(Result2) {
			case 1:	printf("\nWord to Find :");
				/*	Get the word.*/
				fgets(Choice,80,stdin);
				strtok(Choice,"\n\r");
				Length = strlen(Choice);
				/*	Make a pascal string out of the word.	*/
				memmove(Choice+1,Choice,Length);
				Choice[0] = Length;
				/*	Find the word. */
				ixFindKey(IndexManager,&Cursor,(UCharT *)Choice,&Found, &NumRecords, &NumWords, &Status);
				if(Status < 0)
					printf("\nError :%d",Status);
				if(Found == BooleanTrue) {
					printf("\nWord Found");
				}
				else {
					printf("\nWord Not Found");
				}
				break;
			case 2:	
				/*	Display the current key.	*/
				ixCurrentKey(IndexManager,&Cursor,(UCharT *)Choice,&NumRecords, &NumWords,&Status);
				Choice[Choice[0] +1] = 0;
				if(Status < 0)
					printf("\nError :%d",Status);
				printf("\nCurrent Word : %s  NumRecords : %ld, Num Words : %ld",Choice+1, NumRecords, NumWords);
				break;
			case 3:	
				/*	Display the previous key.	*/
				ixPreviousKey(IndexManager,&Cursor, (UCharT *)Choice,&NumRecords, &NumWords,&Status);
				Choice[Choice[0] +1] = 0;
				if(Status < 0)
					printf("\nError :%d",Status);
				printf("\nCurrent Word : %s  NumRecords : %ld, Num Words : %ld",Choice +1,  NumRecords, NumWords);
				break;
			case 4:
				/*	Display the next key.	*/
				ixNextKey(IndexManager,&Cursor, (UCharT *)Choice,&NumRecords, &NumWords,&Status);
				Choice[Choice[0] +1] = 0;
				if(Status < 0)
					printf("\nError :%d",Status);
				printf("\nCurrent Word : %s  NumRecords : %ld, Num Words : %ld",Choice+1,  NumRecords, NumWords);
				break;
			};
			break;
			
			case 9: 
				/*	Display the number of records in the index.	*/
				ixNumberOfRecordsInIndex(IndexManager,&NumRecords,&Status);
				printf("\nThe Number of Records is %ld",NumRecords);
				break;
				
			case 10:
				/*	Rewind a result vector in the results array.*/
				printf("\nVector To Rewind (1-50) :");
				gets(Choice);
				VecPos1 = atoi(Choice);
				ixVectorRewind(Vector[VecPos1],&Status);
				if(Status < 0)
					printf("\nError Rewinding Vector :%d",Status);
				break;
			case 11:
				/*	Get the current hit from the results vector.*/
				printf("\nWhich Vector ? (1-50)");
				gets(Choice);
				VecPos1 = atoi(Choice);
				ixVectorCurrentHit(Vector[VecPos1], &RecordNum, &WordNum, &CountNum, &HitWeight, &Status);
				if(Status < 0) {
					printf("\n Error %d",Status);
					break;
				}
				printf("\nRecord Num = %ld,  WordNum = %ld",RecordNum, WordNum);
				break;
			case 12:
				/*	Get the next hit in the results vector.		*/
				printf("\nWhich Vector ? (1-50)");
				gets(Choice);
				VecPos1 = atoi(Choice);
				ixVectorNextHit(Vector[VecPos1], &RecordNum, &WordNum, &CountNum, &HitWeight, &Status);
				if(Status < 0) {
					printf("\n Error %d",Status);
					break;
				}
				printf("\nRecord Num = %ld,  WordNum = %ld",RecordNum, WordNum);
				break;
			case 13:
				/*	Get the number of hits within the results list.	*/
				printf("\nWhich Vector? (1-50)");
				fgets(Choice,80,stdin);
				strtok(Choice,"\n\r");
				Result2 = atoi(Choice);
				ixNumHits(Vector[Result2],&NumbHits,&NumRecords,&NumWords,&Status);
				if(Status < 0) {
					printf("\nError :%d",Status);
				}
				printf("\nNumHits = %ld, NumRecords = %ld, NumWords = %ld",NumbHits,NumRecords,NumWords);
				break;
			case 14:		
				/*
					Delete a record number.
				*/
				printf("\nRecord Number To Delete :");
				gets(Choice);
				Result2 = atoi(Choice);
				ixDeleteRecordNum(IndexManager,Result2,&Status);
				printf("\nRecord %ld Deleted");
				break;
				
			case 15:   	
				/*
					Undelete a record number.
				*/
				printf("\nRecord Number To Undelete :");
				gets(Choice);
				Result2 = atoi(Choice);
				ixUnDeleteRecordNum(IndexManager,Result2,&Status);
				printf("\nRecord %ld Undeleted");
				break;

			case 16:		
				/*
					Check to see if a specific record number has
					been deleted.
				*/
				printf("\nRecord Number :");
				gets(Choice);
				Result2 = atoi(Choice);
				Result2 = ixIsRecordDeleted(IndexManager,Result2,&Status);
				if(Result2 == 1)
					printf("\nRecord IS Deleted");
				else
					if(Result2 == 0)
						printf("\nRecord is NOT Deleted");
					else
						printf("\nWeird Value Returned.");
					break;
				
			case 17:
				/*
					This outputs a stats file for the index.
					This file is intended for use with some
					upcoming products from Lextek International.

					To output the stats file, you need to have
					a retrieval session in progress on an opened
					index.
				*/
				printf("\nStats DB File Name :");
				fgets(Choice,80,stdin);
				strtok(Choice,"\n\r");
				ixOutputStatsFile(IndexManager,Choice,&Status);
				if(Status == 0) {
					printf("\nStats File Output Succesfully");
				}
				if(Status < 0) {
					printf("\nError %d Outputting Stats File",Status);
				}
				break;
				
			case 18:    
				/*
					Get the results vector associated with the
					current word in the wordlist cursor.
				*/
				printf("\nPlace Query Vector Where ? (1-50) :");
				fgets(Choice,80,stdin);
				strtok(Choice,"\n\r");
				Result2 = atoi(Choice);
				Vector[Result2] = ixGetCurrentQueryVector(IndexManager,&Cursor,&Status);
				break;

			case 19:		
				/*
					Find a record ID from a record number.  In order for this to
					work, the record IDs must have been stored.
				*/
				printf("\nRecord Number Of ID To Get :");
				fgets(Choice,80,stdin);
				strtok(Choice,"\n\r");
				Result2 = atol(Choice);
				if(Result2 == 0)
					Result2 = 1;
				ixRetrieveRecordID(IndexManager,(unsigned char *)ScratchBuff,80,&BuffOffset,(unsigned char *) HexQueryString,Result2,&Status);
				ScratchBuff[BuffOffset] = 0x00;
				printf("\nRecord ID For Record %d Is: %s",Result2,ScratchBuff);
				break;
			case 20:
				/*
					Find a record number from a record ID.  In order for this
					to work, the record IDs must have been indexed.
				*/
				printf("\nRecord ID :");
				fgets(Choice,80,stdin);
				strtok(Choice,"\n\r");
				strcpy(HexQueryString,"0x");
				for(Counter = 0; Choice[Counter] != 0x00; Counter ++) {
					ixCharToHex(Choice[Counter],(unsigned char *)ScratchBuff);
					ScratchBuff[2] = 0x00;
					strcat(HexQueryString,ScratchBuff);
				}
				
				ixFindRecordID(IndexManager,(unsigned char *)HexQueryString,&RecordNum,&Status);
				printf("\nRecord Number Is :%ld",RecordNum);
				break;				
			case 21:		
				/*
					Retrieve text associated with a given record.
				*/
				printf("\nRecord Number of Text :");
				fgets(Choice,80,stdin);
				strtok(Choice,"\n\r");
				Result2 = atoi(Choice);
				ixRetrieveRecordData(IndexManager,(unsigned char *)ScratchBuff,80,&BuffOffset,Result2,&DataCursor,&Status);
				ScratchBuff[BuffOffset] = 0;
				printf("\nRecord Text :%s",ScratchBuff);
				if(DataCursor.Done == BooleanTrue)
					printf("\nRecord Done");
				else
					printf("\nRecord Not Done");
				break;
			case 22:    
				/*
					Retrieve more record text with a given record.  Note, you
					need to call the above command to get record text first
					to get the first chunk of text with a record.
				*/
				ixRetrieveMoreRecordData(IndexManager,(unsigned char *)ScratchBuff,80,&BuffOffset,&DataCursor,&Status);
				ScratchBuff[BuffOffset] = 0;
				printf("\nRecord Text :%s",ScratchBuff);
				if(DataCursor.Done == BooleanTrue)
					printf("\nRecord Done");
				else
					printf("\nRecord Not Done");
				break;
			
			case 23:		
				/*
					This returns the status message associated with the current error code.
				*/
				printf("\nWhat is the error code :");
				fgets(Choice,80,stdin);
				strtok(Choice,"\n\r");
				Result2 = atoi(Choice);
				ixGetStatusMessage((StatusCodeT*) &Result2,Choice, 80);
				printf("\n%s",Choice);
				break;				
			
			case 24:		
				/*
					This returns the amount of temporary disk space currently
					used by the indexing process.  This does not work after a
					call to ixEndIndexingSession() or ixFinalProcessIndex().
					However, it will let you know how much disk space is used.
					You should not allow the amount of disk space used to be more
					than 1/2 of the available disk space.
				*/
				ixGetTempDiskSpaceUsage(Indexer, &TempFileLength,&Status);
				printf("\nTemp File Disk Usage :%ld",TempFileLength);
				break;
			
			
			case 0:
				free(Buffer1);
				free(Buffer2);
				free(Buffer3);
				delete IndexManager;
				exit(1);
		}; // End switch()
	} // end for(;;)
	return 1;
}



